@@ -1484,6 +1484,70 @@ class SqlServerTestCase(unittest.TestCase):
         self.assertEqual(t[5], 2)       # scale
         self.assertEqual(t[6], True)    # nullable
 
+    def test_cursor_messages_with_print(self):
+        """
+        Ensure the Cursor.messages attribute is handled correctly with a simple PRINT statement.
+        """
+        # self.cursor is used in setUp, hence is not brand new at this point
+        brand_new_cursor = self.cnxn.cursor()
+        self.assertIsNone(brand_new_cursor.messages)
+
+        # SQL Server PRINT statements are never more than 8000 characters
+        # https://docs.microsoft.com/en-us/sql/t-sql/language-elements/print-transact-sql#remarks
+        for msg in ('hello world', 'ABCDEFGHIJ' * 800):
+            self.cursor.execute("PRINT '{}'".format(msg))
+            messages = self.cursor.messages
+            self.assertTrue(type(messages) is list)
+            self.assertEqual(len(messages), 1)
+            self.assertTrue(type(messages[0]) is tuple)
+            self.assertEqual(len(messages[0]), 2)
+            self.assertTrue(type(messages[0][0]) is unicode)
+            self.assertTrue(type(messages[0][1]) is unicode)
+            self.assertEqual('[01000] (0)', messages[0][0])
+            self.assertTrue(messages[0][1].endswith(msg))
+
+    def test_cursor_messages_with_stored_proc(self):
+        """
+        Complex scenario to test the Cursor.messages attribute.
+        """
+        self.cursor.execute("""
+            CREATE OR ALTER PROCEDURE test_cursor_messages AS
+            BEGIN
+                SET NOCOUNT ON;
+                PRINT 'Message 1a';
+                PRINT 'Message 1b';
+                SELECT N'Field 1a' AS F UNION ALL SELECT N'Field 1b';
+                SELECT N'Field 2a' AS F UNION ALL SELECT N'Field 2b';
+                PRINT 'Message 2a';
+                PRINT 'Message 2b';
+            END
+        """)
+        # result set 1
+        self.cursor.execute("EXEC test_cursor_messages")
+        rows = [tuple(r) for r in self.cursor.fetchall()]  # convert pyodbc.Row objects for ease of use
+        self.assertEqual(len(rows), 2)
+        self.assertSequenceEqual(rows, [('Field 1a', ), ('Field 1b', )])
+        self.assertEqual(len(self.cursor.messages), 2)
+        self.assertTrue(self.cursor.messages[0][1].endswith('Message 1a'))
+        self.assertTrue(self.cursor.messages[1][1].endswith('Message 1b'))
+        # result set 2
+        self.assertTrue(self.cursor.nextset())
+        rows = [tuple(r) for r in self.cursor.fetchall()]  # convert pyodbc.Row objects for ease of use
+        self.assertEqual(len(rows), 2)
+        self.assertSequenceEqual(rows, [('Field 2a', ), ('Field 2b', )])
+        self.assertEqual(self.cursor.messages, [])
+        # result set 3
+        self.assertTrue(self.cursor.nextset())
+        with self.assertRaises(pyodbc.ProgrammingError):
+            self.cursor.fetchall()
+        self.assertEqual(len(self.cursor.messages), 2)
+        self.assertTrue(self.cursor.messages[0][1].endswith('Message 2a'))
+        self.assertTrue(self.cursor.messages[1][1].endswith('Message 2b'))
+        # result set 4 (which shouldn't exist)
+        self.assertFalse(self.cursor.nextset())
+        with self.assertRaises(pyodbc.ProgrammingError):
+            self.cursor.fetchall()
+        self.assertEqual(self.cursor.messages, [])
 
     def test_none_param(self):
         "Ensure None can be used for params other than the first"
@@ -1544,7 +1608,8 @@ class SqlServerTestCase(unittest.TestCase):
         self.cursor.execute("create table t1(s varchar(800))")
         def test():
             self.cursor.execute("insert into t1 values (?)", value)
-        self.assertRaises(pyodbc.DataError, test)
+        # different versions of SQL Server generate different errors
+        self.assertRaises((pyodbc.DataError, pyodbc.ProgrammingError), test)
 
     def test_geometry_null_insert(self):
         def convert(value):
@@ -1863,6 +1928,14 @@ class SqlServerTestCase(unittest.TestCase):
                         print("Mismatch at row " + str(r+1) + ", column " + str(c+1) + "; expected:", param_array[r][c] , " received:", result_array[r][c])
                         success = False
 
+        try:
+            result_array = self.cursor.execute("exec SelectTVP ?", [[]]).fetchall()
+            self.assertEqual(result_array, [])
+        except Exception as ex:
+            print("Failed to execute SelectTVP")
+            print("Exception: [" + type(ex).__name__ + "]", ex.args)
+            success = False
+
         self.assertEqual(success, True)
 
 
@@ -1887,9 +1960,10 @@ def main():
     else:
         connection_string = args[0]
 
-    cnxn = pyodbc.connect(connection_string)
-    print_library_info(cnxn)
-    cnxn.close()
+    if options.verbose:
+        cnxn = pyodbc.connect(connection_string)
+        print_library_info(cnxn)
+        cnxn.close()
 
     suite = load_tests(SqlServerTestCase, options.test, connection_string)
 
