@@ -66,7 +66,9 @@ class SqlServerTestCase(unittest.TestCase):
     SMALL_FENCEPOST_SIZES = [ 0, 1, 255, 256, 510, 511, 512, 1023, 1024, 2047, 2048, 4000 ]
     LARGE_FENCEPOST_SIZES = [ 4095, 4096, 4097, 10 * 1024, 20 * 1024 ]
 
-    STR_FENCEPOSTS = [ _generate_test_string(size) for size in SMALL_FENCEPOST_SIZES ]
+    STR_FENCEPOSTS = [_generate_test_string(size) for size in SMALL_FENCEPOST_SIZES]
+    LARGE_STR_FENCEPOSTS = STR_FENCEPOSTS + [_generate_test_string(size) for size in LARGE_FENCEPOST_SIZES]
+
     BYTE_FENCEPOSTS    = [ bytes(s, 'ascii') for s in STR_FENCEPOSTS ]
     IMAGE_FENCEPOSTS   = BYTE_FENCEPOSTS + [ bytes(_generate_test_string(size), 'ascii') for size in LARGE_FENCEPOST_SIZES ]
 
@@ -290,10 +292,15 @@ class SqlServerTestCase(unittest.TestCase):
         """
         The implementation for string, Unicode, and binary tests.
         """
-        assert colsize is None or isinstance(colsize, int), colsize
-        assert colsize is None or (value is None or colsize >= len(value))
+        assert (
+            value is None
+            or
+            colsize == -1 or colsize is None or colsize >= len(value)
+        ), colsize
 
-        if colsize:
+        if colsize == -1:
+            sql = "create table t1(s %s(max))" % sqltype
+        elif colsize:
             sql = "create table t1(s %s(%s))" % (sqltype, colsize)
         else:
             sql = "create table t1(s %s)" % sqltype
@@ -383,6 +390,14 @@ class SqlServerTestCase(unittest.TestCase):
     for value in STR_FENCEPOSTS:
         locals()['test_varchar_%s' % len(value)] = _maketest(value)
 
+    # Generate a test for each fencepost size: test_varchar_0, etc.
+    def _maketest(value):
+        def t(self):
+            self._test_strtype('varchar', value, colsize=-1)
+        return t
+    for value in LARGE_STR_FENCEPOSTS:
+        locals()['test_varchar_max_%s' % len(value)] = _maketest(value)
+
     def test_varchar_many(self):
         self.cursor.execute("create table t1(c1 varchar(300), c2 varchar(300), c3 varchar(300))")
 
@@ -412,6 +427,13 @@ class SqlServerTestCase(unittest.TestCase):
     for value in STR_FENCEPOSTS:
         locals()['test_unicode_%s' % len(value)] = _maketest(value)
 
+    def _maketest(value):
+        def t(self):
+            self._test_strtype('nvarchar', value, colsize=-1)
+        return t
+    for value in LARGE_STR_FENCEPOSTS:
+        locals()['test_unicode_max_%s' % len(value)] = _maketest(value)
+
     def test_unicode_longmax(self):
         # Issue 188:	Segfault when fetching NVARCHAR(MAX) data over 511 bytes
 
@@ -1344,6 +1366,70 @@ class SqlServerTestCase(unittest.TestCase):
         self.assertEqual(t[5], 2)       # scale
         self.assertEqual(t[6], True)    # nullable
 
+    def test_cursor_messages_with_print(self):
+        """
+        Ensure the Cursor.messages attribute is handled correctly with a simple PRINT statement.
+        """
+        # self.cursor is used in setUp, hence is not brand new at this point
+        brand_new_cursor = self.cnxn.cursor()
+        self.assertIsNone(brand_new_cursor.messages)
+
+        # SQL Server PRINT statements are never more than 8000 characters
+        # https://docs.microsoft.com/en-us/sql/t-sql/language-elements/print-transact-sql#remarks
+        for msg in ('hello world', 'ABCDEFGHIJ' * 800):
+            self.cursor.execute("PRINT '{}'".format(msg))
+            messages = self.cursor.messages
+            self.assertTrue(type(messages) is list)
+            self.assertEqual(len(messages), 1)
+            self.assertTrue(type(messages[0]) is tuple)
+            self.assertEqual(len(messages[0]), 2)
+            self.assertTrue(type(messages[0][0]) is str)
+            self.assertTrue(type(messages[0][1]) is str)
+            self.assertEqual('[01000] (0)', messages[0][0])
+            self.assertTrue(messages[0][1].endswith(msg))
+
+    def test_cursor_messages_with_stored_proc(self):
+        """
+        Complex scenario to test the Cursor.messages attribute.
+        """
+        self.cursor.execute("""
+            CREATE OR ALTER PROCEDURE test_cursor_messages AS
+            BEGIN
+                SET NOCOUNT ON;
+                PRINT 'Message 1a';
+                PRINT 'Message 1b';
+                SELECT N'Field 1a' AS F UNION ALL SELECT N'Field 1b';
+                SELECT N'Field 2a' AS F UNION ALL SELECT N'Field 2b';
+                PRINT 'Message 2a';
+                PRINT 'Message 2b';
+            END
+        """)
+        # result set 1
+        self.cursor.execute("EXEC test_cursor_messages")
+        rows = [tuple(r) for r in self.cursor.fetchall()]  # convert pyodbc.Row objects for ease of use
+        self.assertEqual(len(rows), 2)
+        self.assertSequenceEqual(rows, [('Field 1a', ), ('Field 1b', )])
+        self.assertEqual(len(self.cursor.messages), 2)
+        self.assertTrue(self.cursor.messages[0][1].endswith('Message 1a'))
+        self.assertTrue(self.cursor.messages[1][1].endswith('Message 1b'))
+        # result set 2
+        self.assertTrue(self.cursor.nextset())
+        rows = [tuple(r) for r in self.cursor.fetchall()]  # convert pyodbc.Row objects for ease of use
+        self.assertEqual(len(rows), 2)
+        self.assertSequenceEqual(rows, [('Field 2a', ), ('Field 2b', )])
+        self.assertEqual(self.cursor.messages, [])
+        # result set 3
+        self.assertTrue(self.cursor.nextset())
+        with self.assertRaises(pyodbc.ProgrammingError):
+            self.cursor.fetchall()
+        self.assertEqual(len(self.cursor.messages), 2)
+        self.assertTrue(self.cursor.messages[0][1].endswith('Message 2a'))
+        self.assertTrue(self.cursor.messages[1][1].endswith('Message 2b'))
+        # result set 4 (which shouldn't exist)
+        self.assertFalse(self.cursor.nextset())
+        with self.assertRaises(pyodbc.ProgrammingError):
+            self.cursor.fetchall()
+        self.assertEqual(self.cursor.messages, [])
 
     def test_none_param(self):
         "Ensure None can be used for params other than the first"
@@ -1459,7 +1545,8 @@ class SqlServerTestCase(unittest.TestCase):
         self.cursor.execute("create table t1(s varchar(800))")
         def test():
             self.cursor.execute("insert into t1 values (?)", value)
-        self.assertRaises(pyodbc.DataError, test)
+        # different versions of SQL Server generate different errors
+        self.assertRaises((pyodbc.DataError, pyodbc.ProgrammingError), test)
 
     def test_geometry_null_insert(self):
         def convert(value):
@@ -1789,6 +1876,14 @@ class SqlServerTestCase(unittest.TestCase):
                         print("Mismatch at row " + str(r+1) + ", column " + str(c+1) + "; expected:", param_array[r][c] , " received:", result_array[r][c])
                         success = False
 
+        try:
+            result_array = self.cursor.execute("exec SelectTVP ?", [[]]).fetchall()
+            self.assertEqual(result_array, [])
+        except Exception as ex:
+            print("Failed to execute SelectTVP")
+            print("Exception: [" + type(ex).__name__ + "]", ex.args)
+            success = False
+
         self.assertEqual(success, True)
         
 def main():
@@ -1812,9 +1907,10 @@ def main():
     else:
         connection_string = args[0]
 
-    cnxn = pyodbc.connect(connection_string)
-    print_library_info(cnxn)
-    cnxn.close()
+    if options.verbose:
+        cnxn = pyodbc.connect(connection_string)
+        print_library_info(cnxn)
+        cnxn.close()
 
     suite = load_tests(SqlServerTestCase, options.test, connection_string)
 
